#!/usr/bin/env python3

from troposphere import ec2
from troposphere import Output, Export, Sub, GetAtt, Ref

from tropostack.base import InlineConfStack
from tropostack.cli import InlineConfCLI

class EC2Stack(InlineConfStack):
    """
    Single-instance EC2 stack, which assigns a static IP address to the
    instance. Also features a security group, dedicated to the instance/stack.
    Uses a human-friendly AMI path specification rather than AMI ID.

    Args:
      region (str): Region where the stack/instance would be deployed
      instance_type (str): EC2 instance type
      ami_location (str): Qualified path to the AMI (i.e. Source in the UI).
        Example: ``amazon/amzn2-ami-hvm-2.0.20191116.0-x86_64-ebs``
      vpc_id (str): VPC that the instance would be a part of
      subnet_id (str): ID of the subnet where the instance would be deployed
      ssh_key_name (str): SSH Keypair name to be associated with the instance
      private_ip (str): Static IP address of the instance. Must be available
        under the respective Subnet
      access (list of 3-tuples): List of 3 tuples to allow Ingress from,
        formatted as (Protocol, Port, Network Range). Sample value:
        ``[('tcp', 22, '0.0.0.0/0'), ]``
    """

    BASE_NAME = 'ec2-instance'
    CONF = {
        'region': 'eu-west-1',
        'instance_type': 't3.nano',
        'access': [('tcp', 22, '0.0.0.0/0'), ],
        'vpc_id': 'REPLACE-ME',
        'subnet_id': 'REPLACE-ME',
        'ssh_key_name': 'REPLACE-ME',
        'private_ip': 'REPLACE-ME',
        'ami_location': '',
    }

    @property
    def r_ec2_secgroup(self):
        return ec2.SecurityGroup(
            "Ec2SecurityGroup",
            VpcId=self.conf['vpc_id'],
            GroupDescription="Access to the instance ports",
            SecurityGroupIngress=[
                ec2.SecurityGroupRule(
                    IpProtocol=proto,
                    FromPort=port,
                    ToPort=port,
                    CidrIp=cidr,
                ) for (proto, port, cidr) in self.conf['access']
            ],
        )

    @property
    def r_ec2(self):
        return ec2.Instance(
            "Ec2Instance",
            ImageId=self.ami_by_location(self.conf['ami_location']),
            InstanceType=self.conf['instance_type'],
            KeyName=self.conf['ssh_key_name'],
            SecurityGroupIds=[self.r_ec2_secgroup.ref()],
            PrivateIpAddress=self.conf['private_ip'],
            SubnetId = self.conf['subnet_id']
        )

if __name__ == '__main__':
    # Wrap the stack in a CLI and run it
    cli = InlineConfCLI(EC2Stack)
    cli.run()


"""
Configuration loading routines
"""
from collections.abc import Mapping

import yaml

from tropostack.exceptions import ConfigLoadError


def partitioned_yaml_loader(fhanlde, stack_basename):
    """
    This loader expects a file handle  to parse YAML from. The root of the YAML
    document must be a mapping. The top-level keys with simple (non-mapping)
    values would be returned, updated with the value of a mapping named the same
    as `stack_basename`

    Args:
        fhandle (file): A file-handle-compatible stream
        stack_basename (str): Base name of the stack to be configured

    Returns:
        dict: Flattened configuration dict pertinent to the specific stack

    Raises:
        tropostack.exceptions.ConfigLoadError: When configuraiton parsing fails

    Usage:
    >>> import StringIO
    >>> config_src = StringIO(u'''
    ...  env: dev
    ...  region: eu-west-1
    ...  stack-foo:
    ...      stackvar: baz
    ...  stack-bar:
    ...      stackvar: bar
    ... ''')
    >>> partitioned_yaml_loader(config_src, 'stack-bar') ==  \\
    ...   {'env': 'dev', 'region': 'eu-west-1', 'stackvar': 'bar'}
    True


    """
    try:
        full_conf = yaml.safe_load(fhanlde)
    except yaml.YAMLError as exc:
        raise ConfigLoadError('YAML Parsing failed: %s' % exc)
    stack_conf = {}
    if not isinstance(full_conf, Mapping):
        raise ConfigLoadError('Top-level configuration must be a mapping')
    for key, value in full_conf.items():
        if not isinstance(value, Mapping):
            # Only non-mapping values get "inherited"
            stack_conf[key] = value
    stack_tree = full_conf.get(stack_basename, {})
    if not isinstance(stack_tree, Mapping):
        raise ConfigLoadError('Stack-specific config must be a mapping')
    stack_conf.update(stack_tree)
    return stack_conf



#!/usr/bin/env python
import io
import os
import re

from setuptools import find_packages
from setuptools import setup
from distutils.util import convert_path

def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding='utf-8') as fd:
        return re.sub(text_type(r':[a-z]+:`~?(.*?)`'), text_type(r'``\1``'), fd.read())

def load_pkg_init():
    result = {}
    init_path = convert_path('tropostack/__init__.py')
    with open(init_path) as init_file:
        exec(init_file.read(), result)
    return result
PKG_INIT = load_pkg_init()

setup(
    name="tropostack",
    version=PKG_INIT['version'],
    url="https://github.com/topostack/tropostack",
    license='MIT',
    author="tie",
    author_email="tropostack@morp.org",
    description=PKG_INIT['__doc__'].strip(),
    long_description_content_type='text/x-rst',
    long_description=read("README.rst"),
    packages=find_packages(exclude=('tests',)),
    install_requires=[
        'boto3',
        'tabulate',
        'troposphere',
        'pyyaml',
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
